library ac_dart.host_event_model;

import 'package:atlassian_connect_host/host_model.dart';
export 'package:atlassian_connect_host/host_model.dart';

import 'package:constrain/constrain.dart';
import 'package:stuff/stuff.dart';

class AcHostEventType {
  final int ordinal;
  final String literal;
  const AcHostEventType(this.ordinal, this.literal);

  static const AcHostEventType installed =
      const AcHostEventType(1, 'installed');
  static final Map<String, AcHostEventType> _eventTypes = {
    installed.literal: installed
  };

  static AcHostEventType fromLiteral(String literal) => _eventTypes[literal];

  String toString() => literal;
}

abstract class AcHostEvent {
  @NotNull()
  final AcHostEventType eventType;

  AcHostEvent(this.eventType);

  Map toJson() => {'eventType': eventType.toString()};
}

class InstallationEvent extends AcHostEvent {
  @NotNull()
  final AcHostRegistration hostRegistration;

  InstallationEvent(this.hostRegistration) : super(AcHostEventType.installed);

  InstallationEvent._fromJson(JsonParser p) : this(
          (new AcHostRegistration.build(
              sharedSecret: p.single('sharedSecret'),
              hostDetails: (new AcHostDetails.build(
                  key: p.single('clientKey'),
                  baseUrl: p.single('baseUrl', (v) => Uri.parse(v)),
                  productType: p.single(
                      'productType', (v) => productTypeFromLiteral(v)),
                  publicKey: p.single('publicKey'))))));

  InstallationEvent.fromJson(Map<String, Object> json)
      : this._fromJson(parseJson(json));

  Map toJson() => (buildJson
    ..add('sharedSecret', hostRegistration.sharedSecret)
    ..add('clientKey', hostRegistration.hostDetails.key)
    ..add('baseUrl', hostRegistration.hostDetails.baseUrl)
    ..add('productType', hostRegistration.hostDetails.productType,
        (t) => literalForProductType(t))
    ..add('publicKey', hostRegistration.hostDetails.publicKey)).json;

  String toString() => 'InstallationEvent[$hostRegistration]';
}
