library ac_dart.descriptor_model.bitbucket.scopes;

import '../descriptor_model.dart';

class BitbucketScope extends Scope {
  BitbucketScope._(String key) : super(key);

  BitbucketScope._std(String group, [String name])
      : this._(name != null ? '$group:$name' : group);

  BitbucketScope._basic(String group) : this._std(group);

  BitbucketScope._write(String group) : this._std(group, 'write');

  BitbucketScope._admin(String group) : this._std(group, 'admin');
}

final _BitbucketScopes bitbucketScopes = new _BitbucketScopes._();

const String _scopePrefix = 'org.bitbucket.';

class _BitbucketScopes {
  final _RepoScopes repository = new _RepoScopes._();
  final _IssueScopes issue = new _IssueScopes._();
  final _AccountScopes account = new _AccountScopes._();
  final _EmailScopes email = new _EmailScopes._();
  final _PullRequestScopes pullRequest = new _PullRequestScopes._();
  final _SnippetScopes snippet = new _SnippetScopes._();
  final _TeamScopes team = new _TeamScopes._();
  final _WebhookScopes webhook = new _WebhookScopes._();
  final _WikiScopes wiki = new _WikiScopes._();

  _BitbucketScopes._();
}

class _RepoScopes {
  static const String repository = 'repository';
  final BitbucketScope basic = new BitbucketScope._basic(repository);
  final BitbucketScope write = new BitbucketScope._write(repository);
  final BitbucketScope admin = new BitbucketScope._admin(repository);

  _RepoScopes._();
}

class _IssueScopes {
  static const String issue = 'issue';
  final BitbucketScope basic = new BitbucketScope._basic(issue);
  final BitbucketScope write = new BitbucketScope._write(issue);

  _IssueScopes._();
}

class _AccountScopes {
  static const String account = 'account';
  final BitbucketScope basic = new BitbucketScope._basic(account);
  final BitbucketScope write = new BitbucketScope._write(account);

  _AccountScopes._();
}

class _SnippetScopes {
  static const String snippet = 'snippet';
  final BitbucketScope basic = new BitbucketScope._basic(snippet);
  final BitbucketScope write = new BitbucketScope._write(snippet);

  _SnippetScopes._();
}

class _PullRequestScopes {
  static const String pullRequest = 'pullrequest';
  final BitbucketScope basic = new BitbucketScope._basic(pullRequest);
  final BitbucketScope write = new BitbucketScope._write(pullRequest);

  _PullRequestScopes._();
}

class _TeamScopes {
  static const String team = 'team';
  final BitbucketScope basic = new BitbucketScope._basic(team);
  final BitbucketScope write = new BitbucketScope._write(team);

  _TeamScopes._();
}


class _WikiScopes {
  static const String wiki = 'wiki';
  final BitbucketScope basic = new BitbucketScope._basic(wiki);

  _WikiScopes._();
}

class _WebhookScopes {
  static const String webhook = 'webhook';
  final BitbucketScope basic = new BitbucketScope._basic(webhook);

  _WebhookScopes._();
}

class _EmailScopes {
  static const String email = 'email';
  final BitbucketScope basic = new BitbucketScope._basic(email);

  _EmailScopes._();
}
