library ac_dart.host_registration_persistence.datastore;

import 'package:atlassian_connect_host/host_registration_persistence.dart';

import 'package:gcloud/db.dart';

import 'package:option/option.dart';
import 'package:logging/logging.dart';
import '../preconditions.dart';
import 'model.dart';

final Logger _log = new Logger('host_registration_persistence.datastore');

typedef DatastoreDB DatastoreDBProvider();

class AcHostRegistrationDatastoreRepository
    implements AcHostRegistrationRepository {
  final DatastoreDBProvider _dbProvider;

  DatastoreDB get _db => _dbProvider();

  // We will place all AcHostRegistrationRecord entries under this key.
  // This allows us to do
  // strongly consistent ancestor queries as opposed to eventually consistent
  // queries.
  Key get _rootKey => _db.emptyKey.append(AcHostRegistrationRecord, id: '1');

  Key _hostEntityKey(String hostKey) =>
      _rootKey.append(AcHostRegistrationRecord, id: hostKey);

  AcHostRegistrationDatastoreRepository(this._dbProvider) {
    checkNotNull(_dbProvider);
  }

  @override
  Future<AcHostRegistration> create(AcHostRegistration hostRegistration) async {
    _log.finer('create ${hostRegistration}');
    final entity = new AcHostRegistrationRecord.fromRegistration(
        hostRegistration, _rootKey);
    _log.finest(
        'saving record ${entity.key.id}/${entity.key.parent} of type  ${entity.key.id.runtimeType}');

    await _doInsert(entity);

    return hostRegistration;
  }

  @override
  Future<Option<AcHostRegistration>> findRegistrationByHostKey(
      String hostKey) async {
    _log.finer('find $hostKey');

    final registrationOpt = await _findAcHostRegistrationRecord(hostKey);
    return registrationOpt.map((b) => b.registration);
  }

  Future _doInsert(Model entity) async {
    try {
      await _db.commit(inserts: [entity]);
    } catch (e, stacktrace) {
      _log.shout(e, stacktrace); // TODO: temporary only
      throw e;
    }
  }

  Future<Option<AcHostRegistrationRecord>> _findAcHostRegistrationRecord(
      String hostKey) async {
    var hostEntityKey = _hostEntityKey(hostKey);
    print(
        'host entity key is $hostEntityKey: ${hostEntityKey.id}; ${hostEntityKey.parent}');

    final registrations = await _db.lookup([hostEntityKey]);
    _log.finest('found registrations $registrations');

    if (registrations == null) {
      return const None();
    }
    return new Option(registrations
        .firstWhere((m) => m is AcHostRegistrationRecord, orElse: () => null));
  }
}

//AcHostRegistrationRepositoryFactory createMongoAcHostRegistrationRepository(
//    String dbUrl) {
//  return () => new AcHostRegistrationDatastoreRepository(new Db(dbUrl));
//}
