library ac_dart.host_registration_persistence.test;

import 'package:atlassian_connect_host/host_model.dart';
import 'dart:async';
import 'package:logging/logging.dart';
export 'package:atlassian_connect_host/host_model.dart' show AcHostRegistration;
export 'dart:async' show Future;
import 'package:option/option.dart';
import 'package:atlassian_connect_host/host_registration_persistence.dart';
import 'dart:io';
import 'dart:convert';
export 'package:option/option.dart';

final Logger _log = new Logger('AcHostRegistrationRepository.test');

class AcHostRegistrationInMemoryTestRepository
    implements AcHostRegistrationRepository {
  final Map<String, AcHostRegistration> _repo = {};

  @override
  Future<AcHostRegistration> create(AcHostRegistration hostRegistration) {
    _repo[hostRegistration.hostDetails.key] = hostRegistration;
    _log.fine('persisted $hostRegistration');
    return new Future.value(hostRegistration);
  }

  @override
  Future<Option<AcHostRegistration>> findRegistrationByHostKey(
          String hostKey) =>
      new Future.value(new Option(_repo[hostKey]));
}

AcHostRegistrationRepositoryFactory createInMemoryAcHostRegistrationRepository() {
  return () => new AcHostRegistrationInMemoryTestRepository();
}

class AcHostRegistrationLocalFileTestRepository
    implements AcHostRegistrationRepository {
  final File _repoFile;

  AcHostRegistrationLocalFileTestRepository(this._repoFile) {
    _log.fine('created test file repo for registrations: $_repoFile');
  }

  @override
  Future<AcHostRegistration> create(AcHostRegistration hostRegistration) async {
    _log.fine('create $hostRegistration');
    final _repo = await _loadRegistrations();
    _repo[hostRegistration.hostDetails.key] = hostRegistration;
    await _saveRegistrations(_repo);
    _log.fine('persisted $hostRegistration');
    return hostRegistration;
  }

  @override
  Future<Option<AcHostRegistration>> findRegistrationByHostKey(
          String hostKey) async =>
      new Option((await _loadRegistrations())[hostKey]);

  Future<Map<String, AcHostRegistration>> _loadRegistrations() async {
    _log.finest('_loadRegistrations');
    if (!await _repoFile.exists()) {
      return {};
    }

    try {
      final jsonStr = await _repoFile.readAsString();
      final Iterable<Map> json = JSON.decode(jsonStr);
      final registrations = json.map((j) => new AcHostRegistration.fromJson(j));

      return new Map.fromIterable(registrations,
          key: (AcHostRegistration r) => r.hostDetails.key);
    } catch (e, stackTrace) {
      _log.shout('failed to load registrations from file $_repoFile - $e', e,
          stackTrace);
      throw e;
    }
  }

  Future _saveRegistrations(Map<String, AcHostRegistration> registrations) {
    _log.finest('_saveRegistrations: $registrations');
    try {
      return _repoFile
          .writeAsString(JSON.encode(registrations.values.toList()));
    } catch (e, stackTrace) {
      _log.shout('failed to save registrations $_repoFile - $e', e, stackTrace);
      throw e;
    }
  }
}
